#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

from doorpi.keyboard.AbstractBaseClass import KeyboardAbstractBaseClass, HIGH_LEVEL, LOW_LEVEL
import doorpi

def get(**kwargs): return DUMMY(**kwargs)
class DUMMY(KeyboardAbstractBaseClass):
    name = 'DUMMY Keyboard'

    def __init__(self, input_pins, output_pins, keyboard_name, bouncetime = 200, polarity = 0, *args, **kwargs):
        logger.debug("__init__(input_pins = %s, output_pins = %s, bouncetime = %s, polarity = %s)",
                     input_pins, output_pins, bouncetime, polarity)

        conf_pre = kwargs['conf_pre']
        conf_post = kwargs['conf_post']

        self.keyboard_name = keyboard_name
        self._polarity = polarity
        self._InputPins = doorpi.DoorPi().config.get_keys(conf_pre+'InputPins'+conf_post)
        self._OutputPins = doorpi.DoorPi().config.get_keys(conf_pre+'OutputPins'+conf_post)

        for output_pin in self._OutputPins:
            self.set_output(output_pin, 0, False)

        self.register_destroy_action()

    def destroy(self):
        if self.is_destroyed: return

        logger.debug("destroy")
        # shutdown all output-pins
        for output_pin in self._OutputPins:
            self.set_output(output_pin, 0, False)
        doorpi.DoorPi().event_handler.unregister_source(__name__, True)
        self.__destroyed = True

    def status_input(self, pin):
        if self._polarity is 0:
            return str(0).lower() in HIGH_LEVEL
        else:
            return str(0).lower() in LOW_LEVEL

    def set_output(self, pin, value, log_output = True):
        parsed_pin = doorpi.DoorPi().parse_string("!"+str(pin)+"!")
        if parsed_pin != "!"+str(pin)+"!":
            pin = parsed_pin

        value = str(value).lower() in HIGH_LEVEL
        if self._polarity is 1: value = not value
        log_output = str(log_output).lower() in HIGH_LEVEL

        if pin not in self._OutputPins: return False
        if log_output: logger.debug("out(pin = %s, value = %s, log_output = %s)", pin, value, log_output)

        self._OutputStatus[pin] = value
        return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import doorpi
from doorpi.action.base import SingleAction

def call_phonenumber_from_file(filename):
    try:
        with open(doorpi.DoorPi().parse_string(filename), 'r') as f:
            phonenumber = f.readline().strip(' \t\n\r')

        logger.debug("firing sipphone.call for this number: %s", phonenumber)
        doorpi.DoorPi().sipphone.call(phonenumber)
        logger.debug("finished sipphone.call for this number: %s", phonenumber)

    except Exception as ex:
        logger.exception("couldn't get phonenumber from file (%s)", ex)
        return False
    return True

def get(parameters):
    return CallPhoneNumberFromFileAction(call_phonenumber_from_file, filename = parameters)

class CallPhoneNumberFromFileAction(SingleAction):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

SIPPHONE_SECTION = 'SIP-Phone'

class SipphoneAbstractBaseClass(object):

    def thread_register(self, name): pass

    @property
    def name(self): return 'SipphoneAbstractBaseClass'

    @property
    def sound_codecs(self): return []

    @property
    def sound_devices(self): return []

    @property
    def current_call(self): return None

    @property
    def video_codecs(self): return []

    @property
    def video_devices(self): return []

    @property
    def current_call_dump(self): return {}

    def __init__(self): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def config(self, **kwargs): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def start(self): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def stop(self): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def destroy(self): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def call(self, number): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def hangup(self): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def is_admin_number(self, number): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def __del__(self): self.destroy()

class RecorderAbstractBaseClass(object):

    def __init__(self): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def config(self, **kwargs): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def start(self): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def stop(self): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def destroy(self): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def __del__(self): self.destroy()

class PlayerAbstractBaseClass(object):

    def __init__(self): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def config(self, **kwargs): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def start(self): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def stop(self): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def destroy(self): raise NotImplementedError("Subclass %s should implement this!"%self.__class__.__name__)
    def __del__(self): self.destroy()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import doorpi
import pjsua as pj

SIPPHONE_SECTION = 'SIP-Phone'
conf = doorpi.DoorPi().config

def call_timeout():
    return conf.get_int(SIPPHONE_SECTION, 'call_timeout', 30)

def max_call_time():
    return conf.get_int(SIPPHONE_SECTION, 'max_call_time', 120)

def sipphone_server():
    return conf.get(SIPPHONE_SECTION, 'server')

def pj_log(level, msg, length):
    # beautify output - remove date / time an line break
    length_date_and_time = len('15:22:35.695 ')
    length_linebreak = len('\n')
    msg = str(msg[length_date_and_time:-length_linebreak])
    # now it's a beautiful msg
    if level == 4: logger.trace("PJ: %s",msg)
    if level == 3: logger.debug("PJ: %s",msg)
    if level == 2: logger.info("PJ: %s",msg)
    if level == 1: logger.warning("PJ: %s",msg)
    if level == 0: logger.error("PJ: %s",msg)

def create_UAConfig():
    logger.debug("create_UAConfig")
    # Doc: http://www.pjsip.org/python/pjsua.htm#UAConfig
    UAConfig = pj.UAConfig()
    UAConfig.max_calls = conf.get_int(SIPPHONE_SECTION, 'ua.max_calls', 1)
    UAConfig.nameserver = conf.get_list(SIPPHONE_SECTION, 'ua.nameserver', [])
    UAConfig.stun_domain = conf.get(SIPPHONE_SECTION, 'ua.stun_domain', '')
    UAConfig.stun_host = conf.get(SIPPHONE_SECTION, 'ua.stun_host', '')
    UAConfig.user_agent = 'DoorPi'
    return UAConfig

def create_MediaConfig():
    logger.debug("create_MediaConfig")
    # Doc: http://www.pjsip.org/python/pjsua.htm#MediaConfig
    MediaConfig = pj.MediaConfig()
    MediaConfig.audio_frame_ptime = conf.get_int(SIPPHONE_SECTION, 'media.audio_frame_ptime', 20)
    MediaConfig.channel_count = conf.get_int(SIPPHONE_SECTION, 'media.channel_count', 1)
    MediaConfig.clock_rate = conf.get_int(SIPPHONE_SECTION, 'media.clock_rate', 8000)
    MediaConfig.ec_options = conf.get_int(SIPPHONE_SECTION, 'media.ec_options', 0)
    MediaConfig.ec_tail_len = conf.get_int(SIPPHONE_SECTION, 'media.ec_tail_len', 1024)
    MediaConfig.enable_ice = conf.get_bool(SIPPHONE_SECTION, 'media.enable_ice', True)
    MediaConfig.enable_turn = conf.get_bool(SIPPHONE_SECTION, 'media.enable_turn', False)
    MediaConfig.ilbc_mode = conf.get_int(SIPPHONE_SECTION, 'media.ilbc_mode', 30)
    MediaConfig.jb_max = conf.get_int(SIPPHONE_SECTION, 'media.jb_max', -1)
    MediaConfig.jb_min = conf.get_int(SIPPHONE_SECTION, 'media.jb_min', -1)
    MediaConfig.max_media_ports = conf.get_int(SIPPHONE_SECTION, 'media.max_media_ports', 32)
    MediaConfig.no_vad = conf.get_bool(SIPPHONE_SECTION, 'media.no_vad', False)
    MediaConfig.ptime = conf.get_int(SIPPHONE_SECTION, 'media.ptime', 0)
    MediaConfig.quality = conf.get_int(SIPPHONE_SECTION, 'media.quality', 10)
    MediaConfig.rx_drop_pct = conf.get_int(SIPPHONE_SECTION, 'media.rx_drop_pct', 0)
    MediaConfig.snd_auto_close_time = conf.get_int(SIPPHONE_SECTION, 'media.snd_auto_close_time', 5)
    MediaConfig.snd_clock_rate = conf.get_int(SIPPHONE_SECTION, 'media.snd_clock_rate', 0)
    MediaConfig.turn_conn_type = conf.get_int(SIPPHONE_SECTION, 'media.turn_conn_type', 17)
    #TODO: string to http://www.pjsip.org/python/pjsua.htm#AuthCred
    MediaConfig.turn_cred = None
    MediaConfig.turn_server = conf.get(SIPPHONE_SECTION, 'media.turn_server', '')
    MediaConfig.tx_drop_pct = conf.get_int(SIPPHONE_SECTION, 'media.tx_drop_pct', 0)
    return MediaConfig

def create_LogConfig():
    logger.debug("create_LogConfig")
    # Doc: http://www.pjsip.org/python/pjsua.htm#LogConfig
    LogConfig = pj.LogConfig(
        callback = pj_log,
        level = conf.get_int(SIPPHONE_SECTION, 'log.level', 1),
        console_level = conf.get_int(SIPPHONE_SECTION, 'log.console_level', 1)
    )
    return LogConfig

def create_AccountConfig():
    logger.debug("create_AccountConfig")
    # Doc: http://www.pjsip.org/python/pjsua.htm#AccountConfig
    server = conf.get(SIPPHONE_SECTION, "server")
    username = conf.get(SIPPHONE_SECTION, "username")
    password = conf.get(SIPPHONE_SECTION, "password")
    realm = conf.get(SIPPHONE_SECTION, "realm")

    AccountConfig = pj.AccountConfig()
    AccountConfig.id = "sip:" + username + "@" + server
    AccountConfig.reg_uri = "sip:" + server
    AccountConfig.auth_cred = [ pj.AuthCred(realm, username, password) ]
    AccountConfig.allow_contact_rewrite = conf.get_bool(SIPPHONE_SECTION, 'account.allow_contact_rewrite', 0)
    AccountConfig.reg_timeout = conf.get_int(SIPPHONE_SECTION, 'account.reg_timeout', 10)

    return AccountConfig

def create_TransportConfig():
    logger.debug("create_TransportConfig")
    # Doc: http://www.pjsip.org/python/pjsua.htm#TransportConfig
    TransportConfig = pj.TransportConfig(
        port = conf.get_int(SIPPHONE_SECTION, 'transport.port', 0),
        bound_addr = conf.get(SIPPHONE_SECTION, 'transport.bound_addr', ''),
        public_addr = conf.get(SIPPHONE_SECTION, 'transport.public_addr', '')
    )
    return TransportConfig


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import importlib
import json

DEFAULT_MODULE_ATTR = ['__doc__', '__file__', '__name__', '__package__', '__path__', '__version__']

def check_module_status(module):
    module['is_fulfilled'] = False if module['fulfilled_with_one'] else True
    for module_name in module['libraries'].keys():
        status = {}
        try:
            package = importlib.import_module(module_name)
            content = dir(package)

            for attr in DEFAULT_MODULE_ATTR:
                if attr in content:
                    status[attr.replace('__', '')] = getattr(package, attr) or ''
                else:
                    status[attr.replace('__', '')] = 'unknown'

            status['installed'] = True
            if module['fulfilled_with_one']: module['is_fulfilled'] = True
            status['content'] = content

        except Exception as exp:
            status = {'installed': False, 'error': str(exp)}
            if not module['fulfilled_with_one']: module['is_fulfilled'] = False

        finally:
            module['libraries'][module_name]['status'] = status

    return module

def load_module_status(module_name):
    module = importlib.import_module('doorpi.status.requirements_lib.'+module_name).REQUIREMENT
    return check_module_status(module)

REQUIREMENTS_DOORPI = {
    'config':           load_module_status('req_config'),
    'sipphone':         load_module_status('req_sipphone'),
    'event_handler':    load_module_status('req_event_handler'),
    'webserver':    load_module_status('req_webserver'),
    'keyboard':    load_module_status('req_keyboard'),
    'system':    load_module_status('req_system')
}

def get(*args, **kwargs):
    try:
        if len(kwargs['name']) == 0: kwargs['name'] = ['']
        if len(kwargs['value']) == 0: kwargs['value'] = ['']

        status = {}
        for name_requested in kwargs['name']:
            for possible_name in REQUIREMENTS_DOORPI.keys():
                if name_requested in possible_name:
                    status[possible_name] = REQUIREMENTS_DOORPI[possible_name]

        return status
    except Exception as exp:
        logger.exception(exp)
        return {'Error': 'could not create '+str(__name__)+' object - '+str(exp)}

def is_active(doorpi_object):
    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import pjsua as pj

from time import sleep

from doorpi.sipphone.pjsua_lib.Config import *
from doorpi.sipphone.pjsua_lib.SipPhoneAccountCallBack import SipPhoneAccountCallBack
from doorpi.sipphone.pjsua_lib.SipPhoneCallCallBack import SipPhoneCallCallBack
from doorpi.sipphone.pjsua_lib.Recorder import PjsuaRecorder
from doorpi.sipphone.pjsua_lib.Player import PjsuaPlayer
from AbstractBaseClass import SipphoneAbstractBaseClass

from doorpi import DoorPi

def get(*args, **kwargs): return Pjsua()
class Pjsua(SipphoneAbstractBaseClass):

    @property
    def name(self): return 'PJSUA wrapper'

    @property
    def lib(self): return self.__Lib

    @property
    def recorder(self): return self.__recorder

    @property
    def player(self): return self.__player

    @property
    def sound_devices(self):
        try:
            all_devices = []
            for sound_device in self.lib.enum_snd_dev():
                all_devices.append({
                  'name':       sound_device,
                  'capture':    True if sound_device.input_channels > 0 else False,
                  'record':     True if sound_device.output_channels > 0 else False
                })
            return all_devices
        except: return []

    @property
    def sound_codecs(self):
        try:
            all_codecs = []
            for codec in self.lib.enum_codecs():
                all_codecs.append({
                    'name':         codec.name,
                    'channels':     codec.channel_count,
                    'bitrate':      codec.avg_bps
                })
        except: return []

    @property
    def current_call_dump(self):
        try:
            return {
                'direction':        'incoming' if self.current_call.info().role == 0 else 'outgoing',
                'remote_uri':       self.current_call.info().remote_uri,
                'total_time':       self.current_call.info().call_time,
                'level_incoming':   self.lib.conf_get_signal_level(0)[0],
                'level_outgoing':   self.lib.conf_get_signal_level(0)[1],
                'camera':           False
            }
        except:
            return {}

    def thread_register(self, name): return self.lib.thread_register(name)

    def __init__(self, *args, **kwargs):
        logger.debug("__init__")

        DoorPi().event_handler.register_event('OnSipPhoneCreate', __name__)
        DoorPi().event_handler.register_event('OnSipPhoneStart', __name__)
        DoorPi().event_handler.register_event('OnSipPhoneDestroy', __name__)

        DoorPi().event_handler.register_event('OnSipPhoneRecorderCreate', __name__)
        DoorPi().event_handler.register_event('OnSipPhoneRecorderDestroy', __name__)

        DoorPi().event_handler.register_event('BeforeSipPhoneMakeCall', __name__)
        DoorPi().event_handler.register_event('OnSipPhoneMakeCall', __name__)
        DoorPi().event_handler.register_event('AfterSipPhoneMakeCall', __name__)
        
        DoorPi().event_handler.register_event('OnSipPhoneCallTimeoutNoResponse', __name__)
        DoorPi().event_handler.register_event('OnSipPhoneCallTimeoutMaxCalltime', __name__)

        self.__Lib = None
        self.__account = None
        self.current_callcallback = None
        self.current_account_callback = None
        self.__recorder = None
        self.__player = None

        self.call_timeout = 30

    def start(self):
        DoorPi().event_handler('OnSipPhoneCreate', __name__)
        self.__Lib = pj.Lib.instance()
        if self.__Lib is None: self.__Lib = pj.Lib()

        logger.debug("init Lib")
        self.__Lib.init(
            ua_cfg      = doorpi.sipphone.pjsua_lib.Config.create_UAConfig(),
            media_cfg   = doorpi.sipphone.pjsua_lib.Config.create_MediaConfig(),
            log_cfg     = doorpi.sipphone.pjsua_lib.Config.create_LogConfig()
        )

        logger.debug("init transport")
        transport = self.__Lib.create_transport(
            type        = pj.TransportType.UDP,
            cfg         = doorpi.sipphone.pjsua_lib.Config.create_TransportConfig()
        )
        logger.debug("Listening on: %s",str(transport.info().host))
        logger.debug("Port: %s",str(transport.info().port))

        logger.debug("Lib.start()")
        self.lib.start(0)

        DoorPi().event_handler.register_action(
            event_name      = 'OnTimeTick',
            action_object   = 'pjsip_handle_events:50'
        )

        logger.debug("init Acc")
        self.current_account_callback = SipPhoneAccountCallBack()
        self.__account = self.__Lib.create_account(
            acc_config  = doorpi.sipphone.pjsua_lib.Config.create_AccountConfig(),
            set_default = True,
            cb          = self.current_account_callback
        )

        self.call_timeout = doorpi.sipphone.pjsua_lib.Config.call_timeout()
        self.max_call_time = doorpi.sipphone.pjsua_lib.Config.max_call_time()

        DoorPi().event_handler('OnSipPhoneStart', __name__)

        self.__recorder = PjsuaRecorder()
        self.__player = PjsuaPlayer()

        logger.debug("start successfully")

    def stop(self, timeout = -1):
        if self.current_call is None:
            logger.debug('no call? -> nothing to do to clean up')
            return True
        else:
            call_info = self.current_call.info()
            if call_info.total_time > timeout:
                logger.info('call timeout - call.info().total_time %s', call_info.total_time)
                return self.hangup()
            return True

    def destroy(self):
        logger.debug("destroy")
        DoorPi().event_handler('OnSipPhoneDestroy', __name__)

        if self.lib is not None:
            self.lib.handle_events()
            self.__Lib.destroy()
            self.lib.handle_events()

        try:
            timeout = 0
            while timeout < 5 and self.__Lib is not None:
                sleep(0.1)
                timeout += 0.1
                self.lib.handle_events()

        except:
            DoorPi().event_handler.unregister_source(__name__, True)
            return

    def self_check(self, timeout):
        self.lib.thread_register('pjsip_handle_events')

        self.lib.handle_events(timeout)

        if self.current_call is not None:
            if self.current_call.is_valid() is 0:
                del self.current_callcallback
                self.current_callcallback = None
                del self.current_call
                self.current_call = None

            try:
                if self.current_call.info().call_time == 0 \
                and self.current_call.info().total_time > self.call_timeout:
                    logger.info("call timeout - hangup current call after %s seconds", self.call_timeout)
                    self.current_call.hangup()
                    DoorPi().event_handler('OnSipPhoneCallTimeoutNoResponse', __name__)

                if self.current_call.info().call_time > self.max_call_time:
                    logger.info("max call time reached - hangup current call after %s seconds", self.max_call_time)
                    self.current_call.hangup()
                    DoorPi().event_handler('OnSipPhoneCallTimeoutMaxCalltime', __name__)
            except:
                pass

    def call(self, number):
        DoorPi().event_handler('BeforeSipPhoneMakeCall', __name__, {'number':number})
        logger.debug("call(%s)",str(number))

        self.lib.thread_register('call_theard')

        sip_server = doorpi.sipphone.pjsua_lib.Config.sipphone_server()
        sip_uri = "sip:"+str(number)+"@"+str(sip_server)

        if self.lib.verify_sip_url(sip_uri) is not 0:
            logger.warning("SIP-URI %s is not valid (Errorcode: %s)", sip_uri, self.lib.verify_sip_url(sip_uri))
            return false
        else:
            logger.debug("SIP-URI %s is valid", sip_uri)

        DoorPi().event_handler('OnSipPhoneMakeCall', __name__)
        if not self.current_call or self.current_call.is_valid() is 0:
            lck = self.lib.auto_lock()
            self.current_callcallback = SipPhoneCallCallBack()
            self.current_call = self.__account.make_call(
                sip_uri,
                self.current_callcallback
            )
            del lck

        elif self.current_call.info().remote_uri == sip_uri:
            if self.current_call.info().total_time <= 1:
                logger.debug("same call again while call is running since %s seconds? -> skip", str(self.current_call.info().total_time))
            else:
                logger.debug("press twice with call duration > 1 second? Want to hangup current call? OK...")
                #self.current_call.hangup()
                self.lib.hangup_all()
        else:
            logger.debug("new call needed? hangup old first...")
            try:
                # self.current_call.hangup()
                self.lib.hangup_all()
            except pj.Error as e:
                logger.exception("Exception: %s", str(e))
            self.call(Number)

        DoorPi().event_handler('AfterSipPhoneMakeCall', __name__)
        return self.current_call

    def is_admin_number(self, remote_uri = None):
        logger.debug("is_admin_number (%s)",remote_uri)

        if remote_uri is None:
            if self.current_call is not None:
                remote_uri = self.current_call.info().remote_uri
            else:
                logger.debug("couldn't catch current call - no parameter and no current_call from doorpi itself")
                return False

        possible_admin_numbers = DoorPi().config.get_keys('AdminNumbers')
        for admin_number in possible_admin_numbers:
            if "sip:"+admin_number+"@" in remote_uri:
                logger.debug("%s is an adminnumber", remote_uri)
                return True

        logger.debug("%s is not an adminnumber", remote_uri)
        return False

    def hangup(self):
        if self.current_call:
            logger.debug("Received hangup request, cancelling current call")
            self.lib.hangup_all()
        else:
            logger.debug("Ignoring hangup request as there is no ongoing call")




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

from doorpi.action.base import SingleAction
import doorpi

def pjsip_handle_events(timeout):
    doorpi.DoorPi().sipphone.self_check(timeout)

def get(parameters):
    parameter_list = parameters.split(',')
    if len(parameter_list) is not 1: return None

    timeout = int(parameter_list[0])

    return PjsipHandleEventsAction(pjsip_handle_events, timeout = timeout)

class PjsipHandleEventsAction(SingleAction):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

def get(*args, **kwargs):
    try:
        if len(kwargs['name']) == 0: kwargs['name'] = ['']
        if len(kwargs['value']) == 0: kwargs['value'] = ['']

        filter = kwargs['name'][0]
        try: max_count = int(kwargs['value'][0])
        except: max_count = 100

        return kwargs['DoorPiObject'].event_handler.db.get_event_log_entries(max_count, filter)
    except Exception as exp:
        logger.exception(exp)
        return {'Error': 'could not create '+str(__name__)+' object - '+str(exp)}

def is_active(doorpi_object):
    if len(doorpi_object.event_handler.db.get_event_log_entries(1, '')):
        return True
    else:
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import doorpi
from doorpi.action.base import SingleAction

HIGH_LEVEL = ['1', 'high', 'on', 'true']
LOW_LEVEL = ['0', 'low', 'off', 'false']

class KeyboardDestroyAction(SingleAction): pass

class KeyboardAbstractBaseClass(object):

    def register_destroy_action(self, destroy_function = False):
        if destroy_function is False: destroy_function = self.destroy
        return doorpi.DoorPi().event_handler.register_action('OnShutdown', KeyboardDestroyAction(destroy_function))

    ############## methods to implement ##############
    def __init__(self): raise NotImplementedError("Subclasses should implement this!")
    #def destroy(self): pass #logger.warning("Subclasses should implement this!")
    def self_test(self): pass # optional - raise NotImplementedError("Subclasses should implement this!")
    def status_input(self, pin): raise NotImplementedError("Subclasses should implement this!")
    def set_output(self, pin, value, log_output = True): raise NotImplementedError("Subclasses should implement this!")
    ############ ############ ############ ############

    keyboard_name = ''
    @property
    def keyboard_typ(self): return self.__class__.__name__
    @property
    def name(self):
        if self.keyboard_name is '': return '%s Keyboard' % self.keyboard_typ
        else: return '%s (Typ: %s) Keyboard' % (self.keyboard_name, self.keyboard_typ)

    _InputPins = []
    @property
    def input_pins(self): return self._InputPins
    _OutputPins = []
    @property
    def output_pins(self): return self._OutputPins
    _OutputStatus = {}
    @property
    def output_status(self): return self._OutputStatus
    last_key = None
    #@property
    #def last_key(self): return self._last_key
    @property
    def additional_info(self): return {
        'keyboard_name'       : self.keyboard_name,
        'keyboard_typ'        : self.keyboard_typ,
        'name'                : self.name,
        'pin'                 : self.last_key
    }
    @property
    def pressed_keys(self):
        pressed_keys = []
        for input_pin in self._InputPins:
            if self.status_inputpin(input_pin):
                pressed_keys.append(input_pin)
        return pressed_keys
    @property
    def pressed_key(self):
        pressed_keys = self.pressed_keys()
        if len(pressed_keys) > 0: return pressed_keys[0]
        else: return None

    @property
    def is_destroyed(self): return self.__destroyed
    __destroyed = False

    def status_output(self, pin):
        return self._OutputStatus[pin]

    def _register_EVENTS_for_pin(self, pin, name):
        for event in ['OnKeyPressed', 'OnKeyUp', 'OnKeyDown']:
            doorpi.DoorPi().event_handler.register_event(event, name)
            doorpi.DoorPi().event_handler.register_event(event+'_'+str(pin), name)
            doorpi.DoorPi().event_handler.register_event(event+'_'+self.keyboard_name+'.'+str(pin), name)

    def _fire_EVENT(self, event_name, pin, name):
        if self.keyboard_name == '':
            doorpi.DoorPi().keyboard.last_key = self.last_key = pin
        else:
            doorpi.DoorPi().keyboard.last_key = self.last_key = self.keyboard_name+'.'+str(pin)
        doorpi.DoorPi().event_handler(event_name, name, self.additional_info)
        doorpi.DoorPi().event_handler(event_name+'_'+str(pin), name, self.additional_info)
        doorpi.DoorPi().event_handler(event_name+'_'+self.keyboard_name+'.'+str(pin), name, self.additional_info)

    def _fire_OnKeyUp(self, pin, name): self._fire_EVENT('OnKeyUp', pin, name)
    def _fire_OnKeyDown(self, pin, name): self._fire_EVENT('OnKeyDown', pin, name)
    def _fire_OnKeyPressed(self, pin, name): self._fire_EVENT('OnKeyPressed', pin, name)

    get_input = status_input
    status_inputpin = status_input
    get_output = status_output
    get_last_key = last_key
    which_keys_are_pressed = pressed_keys
    #__del__ = destroy



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)



REQUIREMENT = dict(
    fulfilled_with_one = True,
    text_description = '',
    events = [
        #dict( name = 'Vorlage', description = ''),
    ],
    configuration = [
        #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.'),
    ],
    libraries = dict(
        ConfigParser = dict(
            text_warning =          '',
            text_description =      'Das Config-Modul wird benötigt um alle Einstellungen in einer Datei abspeichern und später wieder laden zu können.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import ConfigParser</code> eingeben wird.',
            text_configuration =    '''Eine Konfiguration als Eintrag in der Konfigurationsdatei macht logischerweise keinen Sinn.
Deshalb kann die zu nutzende Config-Datei als Parameter (--configfile) beim DoorPi Start mitgegeben werden. Beispiel:
<code>sudo /home/DoorPi/doorpi/main.py --configfile /home/DoorPi/conf/doorpi.ini</code>

Wenn der Parameter wegelassen wird, sucht der ConfigParser automatisch nach folgenden Dateien (wobei !BASEPATH! das Home-Verzeichnis von DoorPi ist)
<ol>
    <li>!BASEPATH!/conf/doorpi.ini</li>
    <li>!BASEPATH!/conf/doorpi.cfg</li>
    <li>!BASEPATH!\conf\doorpi.ini</li>
    <li>!BASEPATH!\conf\doorpi.cfg</li>
</ol>

Sollte keine Datei vorhanden sein, so wird mit default-Werten versucht DoorPi zum Laufen zu bringen und die Config-Datei als erster möglicher Eintrag abzuspeichern.
''',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/configparser.html'
            }
        )
    )
)




#!/usr/bin/python


# Stolen from this source - thx to FB36:
# http://code.activestate.com/recipes/578168-sound-generator-using-wav-file/

def generate_dial_tone(filename = 'dialtone.wav', volume = 50):
    # generate wav file containing sine waves
    # FB36 - 20120617
    import math, wave, array
    duration = 3 # seconds
    freq = 440 # of cycles per second (Hz) (frequency of the sine waves)
    data = array.array('h') # signed short integer (-32768 to 32767) data
    sampleRate = 44100 # of samples per second (standard)
    #sampleRate = 2048 # of samples per second (standard)
    numChan = 1 # of channels (1: mono, 2: stereo)
    dataSize = 2 # 2 bytes because of using signed short integers => bit depth = 16
    numSamplesPerCyc = int(sampleRate / freq)
    numSamples = sampleRate * duration
    for i in range(numSamples / 2):
        sample = 32767 * float(volume) / 100
        sample *= math.sin(math.pi * 2 * (i % numSamplesPerCyc) / numSamplesPerCyc)
        data.append(int(sample))
    for i in range(numSamples / 2):
        sample = 0
        data.append(int(sample))
    f = wave.open(filename, 'w')
    f.setparams((numChan, dataSize, sampleRate, numSamples, "NONE", "Uncompressed"))
    f.writeframes(data.tostring())
    f.close()

    return filename


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

REQUIREMENT = dict(
    fulfilled_with_one = True,
    text_description = '''Die Aufgabe von einem Keyboard innerhalb von DoorPi ist es, Eingaben und Ausgaben zu steuern. Keyboard bezieht sich dabei nicht auf die Tastertur, sondern auf ein Brett mit Knöpfen und LEDs.
DoorPi ist in der Lage mehrere Keyboards gleichzeitig zu verwalten. Dazu muss in der Konfiguration zuerst eine Zuordnung getroffen werden, welchen frei ausgedachten aber eindeutigen Namen das Keyboard bekommmt (KeyboardName) und von welchem Typ das Keyboard ist.
Diese Zuordnung findet in einer Konfigurations-Sektion "keyboards" statt. Danach kann jedes Keyboard drei weitere Sektionen besitzen:
<ol>
<li>allgemeine und Keyboard-spezifische Konfigurationsparameter (Sektionsname "[KeyboardName]")</li>
<li>Auflistung der InputPins (Sektionsname "[KeyboardName]_InputPins") - wobei Pin auch eine Zeichenkette sein kann wie im Beispiel vom RFID-Reader</li>
<li>Auflistung der OutputPins (Sektionsname "[KeyboardName]_OutputPins") mit Pinnummer und sprechendem Namen</li>
</ol>

Die OutputPins können später entweder mit der Pinnummer oder dem sprechenden Namen angesprochen werden. Deshalb sollte der sprechende Name eindeutig sein.

Beispiel (alles nach einem # sind Kommentare):
<code>
[keyboards]
virtuelles = filesystem
rfid = rdm6300

[virtuelles]
base_path_input = !BASEPATH!/keyboards/virtuelles/inputpins # Pfad in dem dann die Input-Dateien zu finden sind
base_path_output = !BASEPATH!/keyboards/virtuelles/outputpins # Pfad in dem dann die Output-Dateien zu finden sind
reset_input = true # Ausgangszustand der Eingabe-Datei soll wiederhergestellt werden, wenn Signal erkannt wurde

[virtuelles_InputPins]
klingel = out:tueroeffner:0,1,3 # Wenn diese Datei eine '1' beinhaltet, dann soll ein Ausgane mit dem Namen tueroeffner gesteuert werden

[virtuelles_OutputPins]
ausgang = tueroeffner # die Datei "ausgang" wird Dateisystem angelegt, aber innerhalb von DoorPi mit "tueroeffner" angesprochen
ausgang2 = fensteroeffner # die Datei "ausgang2" wird Dateisystem angelegt, aber innerhalb von DoorPi mit "fensteroeffner" angesprochen

[rfid]
# keine weitere spezielle Konfiguration, da uns die Default-Werte vollkommen ausreichen
# theoretisch könnte die ganze Sektion weggelassen werden, da sie leer ist

[rfid_InputPins]
1234567 = out:tueroeffner,1,0,3
2345678 = out:fensteroeffner,1,0,3

# eine Sektion [rfid_OutputPins] macht keinen Sinn, da ein RFID-Reader nichts ausgeben kann :)
</code>

Ergebnis des Beispiels ist, dass:
<ol>
    <li>zwei Keyboards innerhalb von DoorPi registiert werden - das eine dateibasierend mit dem Namen "virtuell", das andere ein RFID-Reader mit dem Namen "rfid"</li>
    <li>dem virtuellen Keyboard alle nötigen Parameter mitgegeben werden, beim rfid Keyboard nur die Default-Parameter genutzt werden.</li>
    <li>ein InputPin für das virtuelle Keyboard angelegt wird</li>
    <li>wenn die Datei <code>!BASEPATH!/keyboards/virtuelles/inputpins/klingel</code> eine 1 als Inhalt bekommt, wird
        <ul>
            <li>Ausgang tueroeffner (in dem Fall die Datei !BASEPATH!/keyboards/virtuelles/outputpins/ausgang)</li>
            <li>neu geschrieben und bekommt am Anfang den Inhalt 1</li>
            <li>es wird drei Sekunden gewartet</li>
            <li>und die Datei wird neu geschrieben, diesmal mit dem Inhalt 0</li>
        </ul>
    </li>
    <li>ähnliches für das rfid Keyboard definiert wurde:
        <ul>
            <li>der RFID-Chip mit dem Code "1234567" öffnet den tueroeffner</li>
            <li>der RFID-Chip mit dem Code "2345678" öffnet den fensteroeffner</li>
        </ul>
    </li>
</ol>

Wobei !BASEPATH! für das Home-Verzeichnis von DoorPi steht (meistens "/home/DoorPi").
''',
    events = [
        dict( name = 'OnKeyPressed', description = 'Es wurde eine Taste als betätigt gemeldet. Das kann je nach Keyboard OnKeyUp oder OnKeyDown sein.'),
        dict( name = 'OnKeyUp', description = 'Eine Taste wurde wieder losgelassen.'),
        dict( name = 'OnKeyDown', description = 'Eine Taste wurde gedrückt, aber noch nicht wieder los gelassen.'),
        dict( name = 'OnKeyPressed_[PinName]', description = 'Gleich wie OnKeyPressed aber beinhaltet außerdem den Pin-Namen um gezielter mit Actions reagieren zu können.'),
        dict( name = 'OnKeyUp_[PinName]', description = 'Gleich wie OnKeyUp aber beinhaltet außerdem den Pin-Namen um gezielter mit Actions reagieren zu können.'),
        dict( name = 'OnKeyDown_[PinName]', description = 'Gleich wie OnKeyDown aber beinhaltet außerdem den Pin-Namen um gezielter mit Actions reagieren zu können.'),
        dict( name = 'OnKeyPressed_[KeyboardName]_[PinName]', description = 'Gleich wie OnKeyPressed aber beinhaltet außerdem den Keyboard-Namen und den Pin-Namen um ganz exakt mit Actions reagieren zu können.'),
        dict( name = 'OnKeyUp_[KeyboardName]_[PinName]', description = 'Gleich wie OnKeyUp aber beinhaltet außerdem den Keyboard-Namen und den Pin-Namen um ganz exakt mit Actions reagieren zu können.'),
        dict( name = 'OnKeyDown_[KeyboardName]_[PinName]', description = 'Gleich wie OnKeyDown aber beinhaltet außerdem den Keyboard-Namen und den Pin-Namen um ganz exakt mit Actions reagieren zu können.')
    ],
    configuration = [
        dict( section = 'keyboards', key = '*', type = 'string', default = 'dummy', mandatory = False, description = 'In der Sektion werden die genutzten Keyboards mit Namen und Typ im Stil <code>[KeyboardName] = [KeyboardTyp]</code> aufgelistet. Die komplette Sektion wird ausgelesen.'),
        dict( section = '[KeyboardName]', key = 'bouncetime', type = 'float', default = '2000', mandatory = False, description = 'bouncetime ist ein softwareseitiger Prellschutz innerhalb dessen Zeit in ms alle weiteren Ereignisse ignoriert werden.'),
        dict( section = '[KeyboardName]', key = 'polarity', type = 'integer', default = '0', mandatory = False, description = 'polarity verdreht die Logik der Eingänge, so dass HIGH-Pegel = LOW-Pegel und umgedreht. Hat aber nur auf die Eingänge Auswirkung!'),
        dict( section = '[KeyboardName]_InputPins', key = '*', type = 'string', default = '', mandatory = False, description = 'Auflistung der Eingabeschnittstellen im Format <code>[PinName] = [Action]</code>. Bitte dazu die möglichen Actions und deren Syntax beachten!'),
        dict( section = '[KeyboardName]_OutputPins', key = '*', type = 'string', default = '', mandatory = False, description = 'Auflistung der Eingabeschnittstellen im Format <code>[PinName] = [SprechenderPinName]</code> - z.B. gibt es den GPIO Ausgang 27 für den Türöffner, so wäre die Syntax <code>27 = Tueroeffner</code>. Umlaute und Sonderzeichen sollten vermieden werden!')
    ],
    libraries = { # nicht als dict(), da es sonst Probleme mit dem Punkt in RPi.GPIO als Key gibt
        'pifacedigitalio': dict(
            text_warning =          '''Neben der reinen Installation vom Python-Modul pifacedigitalio ist es auch wichtig SPI am System zu aktivieren (siehe Links).
Außerdem muss bei Bestellungen darauf geachtet werden, dass es zwei Versionen gibt (<a href="https://www.rasppishop.de/Piface-Digital-Erweiterung-fuer-Raspberry-Pi">PiFace digital 1</a> und <a href="https://www.rasppishop.de/PiFace-Digital-2-Erweiterungsplatine-/-Modul-fuer-den-Raspberry-Pi-Modell-B-">PiFace digital 2</a>)''',
            text_description =      'Das Python-Modul pifacedigitalio ist der "Treiber" für die PiFace Hardware.',
            text_installation =     '',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import pifacedigitalio</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/configparser.html',
                'PiFace Beschreibung auf piface.org.uk': 'http://www.piface.org.uk/products/piface_digital/',
                'Installationsanleitung auf github': 'http://piface.github.io/pifacedigitalio/installation.html',
                'SPI und I2C aktivieren': 'http://raspberry.tips/faq/raspberry-pi-spi-und-i2c-aktivieren/'
            }
        ),
        'RPi.GPIO': dict(
            text_warning =          '',
            text_description =      'RPi.GPIO kümmert sich um die Ein- und Ausgaben der GPIO Schnittstelle eines Raspberry Pi.',
            text_installation =     'Die Installation ist sehr gut <a href="http://sourceforge.net/p/raspberry-gpio-python/wiki/install/">auf Sourceforge</a> beschrieben.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import RPi.GPIO</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'www.raspberrypi.org': {
                    'GPIO Overview': 'https://www.raspberrypi.org/documentation/hardware/raspberrypi/gpio/README.md',
                    'GPIO Usage': 'https://www.raspberrypi.org/documentation/usage/gpio/README.md'
                },
                'RPi.GPIO on pypi': 'https://pypi.python.org/pypi/RPi.GPIO',
                'Installationsanleitung auf Sourceforge': 'http://sourceforge.net/p/raspberry-gpio-python/wiki/install/'
            }
        ),
        'serial': dict(
            text_warning =          '',
            text_description =      '''
Hier die Beschreibung aus der <a href="https://github.com/motom001/DoorPi/blob/master/doorpi/keyboard/from_rdm6300.py">from_rdm6300.py</a>, die <a href="https://github.com/msmolny">msmolny</a> netterweise erstellt hat.

<pre>
  Configuration
  -------------

  1. Define a new keyboard of type 'rdm6300'
  2. Define inputPins section for that keyboard
  3. Each RFID tag has a decimal number printed 
     on the surface. This is the Input PIN number. 
     Define this number and an appropriate action.

  Sample:

  [keyboards]
  rfidreader = rdm6300
  ...
  [rfidreader_InputPins]
  1234567 = out:Tueroeffner,1,0,3
  2345678 = out:Tueroeffner,1,0,3

  That's all...


  Hardware Connections
  --------------------

  RDM6300 Pin Layout
  +-------------------------+
  |                         |
  | (1) ANT1                |
  | (2) ANT2                |
  | P2                      |
  |                         |
  |                         |
  |                         |
  |                     P1  |
  |             +5V(DC) (5) |
  | P3              GND (4) |
  | (3) GND             (3) |
  | (2) +5V(DC)      RX (2) |
  | (1) LED          TX (1) |
  |                         |
  +-------------------------+

  Connect one of the two +5V(DC) and one of the two GND to 
  5V (Pin 2 on the RaspberryPi Board) and to GND (Pin 6 on 
  the RaspberryPi Board). As I used a ribbon cable, the 
  simplest way was to connect to (4) and (5) of P1 from the RDM6300. 
 
  Then, connect TX (pin (1) of P1) to RXD from the UART (Pin 10 
  on the RaspberryPi Board) - BUT NOT DIRECTLY, OTHERWISE YOU 
  MIGHT DAMAGE YOUR RASPBERRY PI!!!
  The RaspberryPi expects 3,3V level on the UART Pins, but the 
  RDM6300 delivers 5V. 

  Simplest solution for this is a voltage divider via resistors:
     RDM6300 P1(1) <--- Resistor R1 ---> RasPi Board(Pin 10)
     GND           <--- Resistor R2 ---> RasPi Board(Pin 10) 
  Ideal solution: R1=5k, R2=10k, this will deliver exactly 3,3V 
                  to RasPi Board(Pin 10)
  Alternative solution: As most RaspberryPi bundles only contain 
                        10k resistors, you might either use 2 
                        10k resistors in parallel to get a 5k 
                        resistor, or simply use 10k for R1 instead.
                        R1=R2=10k will deliver 2,5V to RasPi 
                        Board(Pin 10), but that works also.

  Reference: I used this resource to learn how to work with RDM6300, 
             how to connect it to the RaspberryPi and how to handle
             RFID data: http://kampis-elektroecke.de/?page_id=3248
</pre>
''',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import serial</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                dict( section = '[KeyboardName]', key = 'port', type = 'string', default = '/dev/ttyAMA0', mandatory = False, description = ''),
                dict( section = '[KeyboardName]', key = 'baudrate', type = 'integer', default = '9600', mandatory = False, description = ''),
                dict( section = '[KeyboardName]', key = 'dismisstime', type = 'integer', default = '5', mandatory = False, description = ''),
            ],
            text_links = {
                'serial @ pypi': 'https://pypi.python.org/pypi/serial'
            }
        ),
        'watchdog': dict(
            text_warning =          'Häufiges Lesen und Schreiben von SD-Karten wie im RPi können deren Verschleiß fördern. Eventuell sollte auf tmpfs Verzeichnisse ausgewichen werden.',
            text_description =      '''Das Python-Modul watchdog wird genutzt um ein dateibasierendes Keyboard zu erstellen.
So können entweder zu Testzwecken ohne Hardware-Aufbau Events und Actions getestet werden oder es kann als Schnittstelle zu anderen Systemen dienen,
die per SSH-Befehle die Dateien schreiben und lesen, die auch vom virtuellen keyboard verarbeitet werden.
Dabei kann eingestellt werden, in welchem Ordner die Dateien liegen, die jeweils als Ein- und Ausgabe fungieren und ob die Eingabe Dateien nach Erkennung eines Events durch das Filesystem-Keyboard wieder zurück in den Ausgangszustand versetzt werden.
''',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import ConfigParser</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                dict( section = '[KeyboardName]', key = 'base_path_input', type = 'string', default = '', mandatory = False, description = 'Der Pfad in dem die Eingangspins angelegt werden'),
                dict( section = '[KeyboardName]', key = 'base_path_output', type = 'string', default = '', mandatory = False, description = 'Der Pfad in dem die Eingangspins angelegt werden'),
                dict( section = '[KeyboardName]', key = 'reset_input', type = 'boolean', default = 'True', mandatory = False, description = 'Gibt an ob die Dateien nach Erkennung eines Events durch das Filesystem-Keyboard wieder zurück in den Ausgangszustand versetzt werden')
            ],
            text_links = {
                'watchdog @ pypi': 'https://pypi.python.org/pypi/watchdog'
            }
        )
    }
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import os
import ntpath
from time import sleep

from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler

from doorpi.keyboard.AbstractBaseClass import KeyboardAbstractBaseClass, HIGH_LEVEL, LOW_LEVEL
import doorpi

def path_leaf(path):
    head, tail = ntpath.split(path)
    return tail or ntpath.basename(head)

class MissingMandatoryParameter(Exception): pass

def get(**kwargs): return FileSystem(**kwargs)
class FileSystem(KeyboardAbstractBaseClass, FileSystemEventHandler):

    __reset_file = None
    
    def __init__(self, input_pins, output_pins, conf_pre, conf_post, keyboard_name, polarity = 0, *args, **kwargs):
        logger.debug("FileSystem.__init__(input_pins = %s, output_pins = %s, polarity = %s)",
                     input_pins, output_pins, polarity)
        self.keyboard_name = keyboard_name
        self._polarity = polarity
        self._InputPins = map(str, input_pins)
        self._OutputPins = map(str, output_pins)

        section_name = conf_pre+'keyboard'+conf_post
        self.__reset_input = doorpi.DoorPi().config.get_bool(section_name, 'reset_input', True)
        self.__base_path_input = doorpi.DoorPi().config.get_string_parsed(section_name, 'base_path_input')
        self.__base_path_output = doorpi.DoorPi().config.get_string_parsed(section_name, 'base_path_output')

        if self.__base_path_input == '': raise MissingMandatoryParameter('base_path_input in %s '%section_name)
        if self.__base_path_output == '': raise MissingMandatoryParameter('base_path_output in %s '%section_name)

        if not os.path.exists(os.path.dirname(self.__base_path_input)):
            logger.info('Path %s does not exist - creating it now', os.path.dirname(self.__base_path_input))
            os.makedirs(os.path.dirname(self.__base_path_input))

        if not os.path.exists(os.path.dirname(self.__base_path_output)):
            logger.info('Path %s does not exist - creating it now', os.path.dirname(self.__base_path_output))
            os.makedirs(os.path.dirname(self.__base_path_output))

        for input_pin in self._InputPins:
            self.__set_input(os.path.join(self.__base_path_input, input_pin))
            self._register_EVENTS_for_pin(input_pin, __name__)

        self.__observer = Observer()
        self.__observer.schedule(self, self.__base_path_input)
        self.__observer.start()

        # use set_output to register status @ dict self.__OutputStatus
        for output_pin in self._OutputPins:
            self.set_output(output_pin, 0, False)

        self.register_destroy_action()

    def destroy(self):
        if self.is_destroyed: return
        logger.debug("destroy")

        self.__observer.stop()
        self.__observer.join()

        for input_pin in self._InputPins:
            os.remove(os.path.join(self.__base_path_input, input_pin))
        for output_pin in self._OutputPins:
            os.remove(os.path.join(self.__base_path_output, output_pin))

        doorpi.DoorPi().event_handler.unregister_source(__name__, True)
        self.__destroyed = True

    def status_input(self, pin):
        f = open(os.path.join(self.__base_path_input, pin), 'r')
        plain_value = f.readline().rstrip()
        f.close()
        if self._polarity is 0:
            return str(plain_value).lower() in HIGH_LEVEL
        else:
            return str(plain_value).lower() in LOW_LEVEL

    def __write_file(self, file, value = False):
        f = open(file, 'w')
        value = str(value).lower() in HIGH_LEVEL
        if self._polarity is 1: value = not value
        f.write(str(value)+'\r\n')
        f.close()
        return value

    def __set_input(self, file, value = False):
        self.__write_file(file, value)
        os.chmod(file, 0o666)

    def set_output(self, pin, value, log_output = True):
        parsed_pin = doorpi.DoorPi().parse_string("!"+str(pin)+"!")
        if parsed_pin != "!"+str(pin)+"!":
            pin = parsed_pin

        value = str(value).lower() in HIGH_LEVEL
        log_output = str(log_output).lower() in HIGH_LEVEL

        if pin not in self._OutputPins: return False
        written_value = self.__write_file(os.path.join(self.__base_path_output, pin), value)
        if log_output: logger.debug("out(pin = %s, value = %s, log_output = %s)", pin, written_value, log_output)

        self._OutputStatus[pin] = value
        return True

    def on_modified(self, event):
        if 'FileModifiedEvent' not in str(event): return
        if self.__reset_file:
            if self.__reset_file == event.src_path:
                self.__reset_file = None
                logging.debug('reset inputfile will not fire event (%s)', event.src_path)
                return
            self.__reset_file = event.src_path
            self.__set_input(event.src_path, 'false')

        input_pin = path_leaf(event.src_path)
        if input_pin not in self._InputPins: return

        if self.status_input(input_pin):
            self.__reset_file = event.src_path
            self._fire_OnKeyPressed(input_pin, __name__)
            self._fire_OnKeyDown(input_pin, __name__)
        else:
            self._fire_OnKeyUp(input_pin, __name__)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import os

from doorpi import DoorPi
from doorpi.sipphone.AbstractBaseClass import RecorderAbstractBaseClass, SIPPHONE_SECTION

class LinphoneRecorder(RecorderAbstractBaseClass):

    __record_filename = ''
    __last_record_filename = ''

    @property
    def record_filename(self): return self.__record_filename

    @property
    def parsed_record_filename(self): return DoorPi().parse_string(self.__record_filename)

    @property
    def last_record_filename(self): return self.__last_record_filename

    def __init__(self):
        self.__record_filename = DoorPi().config.get(SIPPHONE_SECTION, 'records', '')
        if self.__record_filename is '':
            logger.debug('no recorder found in config at section DoorPi and key records')
            return

        DoorPi().event_handler.register_action('OnSipPhoneDestroy', self.destroy)

        DoorPi().event_handler.register_event('OnRecorderStarted', __name__)
        DoorPi().event_handler.register_event('OnRecorderStopped', __name__)
        DoorPi().event_handler.register_event('OnRecorderCreated', __name__)

        if DoorPi().config.get_bool(SIPPHONE_SECTION, 'record_while_dialing', 'False') is True:
            DoorPi().event_handler.register_action('OnSipPhoneMakeCall', self.start)
        else:
            DoorPi().event_handler.register_action('OnCallStateConnect', self.start)

        DoorPi().event_handler.register_action('OnCallStateDisconnect', self.stop)

        DoorPi().event_handler('OnRecorderCreated', __name__)

    def start(self):
        if self.__record_filename is '':
            return

        if self.__record_filename is not '':
            self.__last_record_filename = self.parsed_record_filename
            if not os.path.exists(os.path.dirname(self.__last_record_filename)):
                logger.info('Path %s does not exist - creating it now', os.path.dirname(self.__last_record_filename))
                os.makedirs(os.path.dirname(self.__last_record_filename))

            logger.debug('starting recording to %s', self.__last_record_filename)
            DoorPi().sipphone.current_call.start_recording()
            DoorPi().event_handler('OnRecorderStarted', __name__, {
                'last_record_filename': self.__last_record_filename
            })

    def stop(self):
        if not DoorPi().sipphone.current_call: return
        logger.debug('stopping recording to %s', self.__last_record_filename)
        DoorPi().sipphone.current_call.stop_recording()
        DoorPi().event_handler('OnRecorderStopped', __name__, {
            'last_record_filename': self.__last_record_filename
        })

    def destroy(self):
        try: self.stop()
        except: pass
        DoorPi().event_handler.unregister_source(__name__, True)



# -*- coding: utf-8 -*-
"""Project metadata

Information describing the project.
"""
import os

# The package name, which is also the "UNIX name" for the project.
package = 'DoorPi'
project = "VoIP Door-Intercomstation with Raspberry Pi"
project_no_spaces = project.replace(' ', '')
version = '2.4.0.8'
description = 'provide intercomstation to the doorstation by VoIP'
keywords = ['intercom', 'VoIP', 'doorstation', 'home automation', 'IoT']
authors = ['Thomas Meissner']
authors_emails = emails = ['motom001@gmail.com']
authors_string = ', '.join(authors)
author_strings = []
for name, email in zip(authors, authors_emails):
    author_strings.append('{0} <{1}>'.format(name, email))

supporters = [
    'Phillip Munz <office@businessaccess.info>',
    'Hermann Dötsch <doorpi1@gmail.com>',
    'Dennis Häußler <haeusslerd@outlook.com>',
    'Hubert Nusser <hubsif@gmx.de>',
    'Michael Hauer <frrr@gmx.at>',
    'missing someone? -> sorry -> mail me'
]
supporter_string = '\n'.join(supporters)
copyright = "%s, 2014-2015"%authors[0]
license = 'CC BY-NC 4.0'
url = 'https://github.com/motom001/DoorPi'

# created with: http://patorjk.com/software/taag/#p=display&f=Ogre&t=DoorPi
epilog = '''
    ___                  ___ _
   /   \___   ___  _ __ / _ (_)  {project}
  / /\ / _ \ / _ \| '__/ /_)/ |  version:   {version}
 / /_// (_) | (_) | | / ___/| |  license:   {license}
/___,' \___/ \___/|_| \/    |_|  URL:       <{url}>

Authors:    {authors}
Supporter:  {supporters}
'''.format(
        license = license,
        project = project,
        version = version,
        authors = '\n'.join(author_strings),
        supporters = '\n            '.join(supporters),
        url = url)

import tempfile
base_path = tempfile.gettempdir()
if os.access(base_path, os.W_OK):
    print("USE BASE_PATH1: %s"%base_path)
else:
    base_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    print("USE BASE_PATH2: %s"%base_path)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

from datetime import datetime

def get(*args, **kwargs):
    try:
        if len(kwargs['name']) == 0: kwargs['name'] = ['']
        if len(kwargs['value']) == 0: kwargs['value'] = ['']

        keyboard = kwargs['DoorPiObject'].keyboard

        status = {}

        for name_requested in kwargs['name']:
            if name_requested in 'name':
                status['name'] = keyboard.name

            if name_requested in 'input':
                status['input'] = {}
                for value_requested in kwargs['value']:
                    for input_pin in keyboard.input_pins:
                        if value_requested in input_pin:
                            status['input'][input_pin] = keyboard.status_input(input_pin)

            if name_requested in 'output':
                status['output'] = keyboard.output_status
                for value_requested in kwargs['value']:
                    for output_pin in status['output'].keys():
                        if value_requested not in output_pin:
                            del status['output'][output_pin]
        return status

    except Exception as exp:
        logger.exception(exp)
        return {'Error': 'could not create keyboard object - '+str(exp)}

def is_active(doorpi_object):
    try:
        return True if doorpi_object.keyboard.name else False
    except:
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import threading
import serial 
import time
from os import linesep as OS_LINESEP

from doorpi.keyboard.AbstractBaseClass import KeyboardAbstractBaseClass, HIGH_LEVEL, LOW_LEVEL
import doorpi

CONFIG = doorpi.DoorPi().config

def get(**kwargs): return UsbPlain(**kwargs)
class UsbPlain(KeyboardAbstractBaseClass):
    name = 'UsbPlain Keyboard'

    @property
    def last_received_chars(self): return self._last_received_chars

    _ser = None

    def read_usb_plain(self):

        self._last_received_chars = ""
        while not self._shutdown and self._ser.isOpen():
            # char aus buffer holen
            newChar = self._ser.read()
            if newChar == "": continue

            self._last_received_chars += str(newChar)
            logger.debug("new char %s read and is now %s", newChar, self._last_received_chars)

            for input_pin in self._InputPins:
                if self._last_received_chars.endswith(input_pin):
                    self.last_key = input_pin
                    self._fire_OnKeyDown(input_pin, __name__)
                    self._fire_OnKeyPressed(input_pin, __name__)
                    self._fire_OnKeyUp(input_pin, __name__)

            if self._last_received_chars.endswith(self._input_stop_flag):
                logger.debug("found input stop flag -> clear received chars")
                self._last_received_chars = ""
            if len(self._last_received_chars) > self._input_max_size:
                logger.debug("received chars bigger then input max size -> clear received chars")
                self._last_received_chars = ""

        return

    def __init__(self, input_pins, output_pins, conf_pre, conf_post, keyboard_name, *args, **kwargs):
        logger.debug("FileSystem.__init__(input_pins = %s, output_pins = %s)", input_pins, output_pins)
        self.keyboard_name = keyboard_name
        self._InputPins = map(str, input_pins)
        self._OutputPins = map(str, output_pins)

        self._last_received_chars = ""
        self.last_key = ""

        for input_pin in self._InputPins:
            self._register_EVENTS_for_pin(input_pin, __name__)

        # use set_output to register status @ dict self.__OutputStatus
        for output_pin in self._OutputPins:
            self.set_output(output_pin, 0, False)

        # somit wirds aus der Config-Datei geladen, falls dort vorhanden.
        section_name = conf_pre+'keyboard'+conf_post

        port = CONFIG.get(section_name, 'port', "/dev/ttyUSB0")
        baudrate = CONFIG.get_int(section_name, 'baudrate', 9600)

        self._input_stop_flag = CONFIG.get(section_name, 'input_stop_flag', OS_LINESEP)
        self._input_max_size = CONFIG.get_int(section_name, 'input_max_size', 255)
        self._output_stop_flag = CONFIG.get(section_name, 'output_stop_flag', OS_LINESEP)

        self._ser = serial.Serial(port, baudrate)

        self._ser.timeout = 1             #block read, 0 for #non-block read, > 0 for timeout block read
        self._ser.close()
        #self._ser.bytesize = serial.EIGHTBITS       #number of bits per bytes
        #self._ser.parity = serial.PARITY_NONE       #set parity check: no parity
        #self._ser.stopbits = serial.STOPBITS_ONE    #number of stop bits
        #self._ser.xonxoff = False         #disable software flow control
        #self._ser.rtscts = False          #disable hardware (RTS/CTS) flow control
        #self._ser.dsrdtr = False          #disable hardware (DSR/DTR) flow control
        #self._ser.writeTimeout = 0        #timeout for write

        self._ser.open()

        self._shutdown = False
        self._thread = threading.Thread(target = self.read_usb_plain)
        self._thread.daemon = True
        self._thread.start()

        doorpi.DoorPi().event_handler.register_action('OnShutdown', self.destroy)

    def destroy(self):
        if self.is_destroyed: return
        logger.debug("destroy")
        self._shutdown = True
        if self._ser and self._ser.isOpen(): self._ser.close()
        doorpi.DoorPi().event_handler.unregister_source(__name__, True)
        self.__destroyed = True
        return

    def status_input(self, input_pin):
        logger.debug("status_input for tag %s", tag)
        if input_pin == self.last_key:
            return True
        else:
            return False

    def set_output(self, pin, value, log_output = True):
        if self._ser and self._ser.isOpen():
            if log_output: logger.debug('try to write %s to serial usb plain', pin)
            self._ser.flushOutput()
            self._ser.write(pin + self._output_stop_flag)
            self._ser.flush()
            return True
        else:
            if log_output: logger.warning("couldn't write to serial usb plain, because it's not open")
            return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

from doorpi.action.base import SingleAction
import doorpi
from doorpi.status.status_class import DoorPiStatus

def write_statusfile(filename, filecontent):
    try:
        filename = doorpi.DoorPi().parse_string(filename)
        filecontent = doorpi.DoorPi().parse_string(filecontent)

        try:
            doorpi_status = DoorPiStatus(doorpi.DoorPi())
            doorpi_status_json_beautified = doorpi_status.json_beautified
            doorpi_status_json = doorpi_status.json

            filecontent = filecontent.replace('!DOORPI_STATUS.json_beautified!', doorpi_status_json_beautified)
            filecontent = filecontent.replace('!DOORPI_STATUS.json!', doorpi_status_json)

        except:
            logger.exception("error while creating status")

    except:
        logger.warning("while action statusfile - error to get DoorPi().parse_string")
        return False

    try:
        file = open(filename, 'w')
        try:
            file.write(filecontent)
            file.flush()
        finally:
            file.close()
    except IOError as e:
        logger.warning("while action statusfile - I/O error(%s): %s"%(e.errno, e.strerror))
        return False

    return True

def get(parameters):
    parameter_list = parameters.split(',')
    if len(parameter_list) < 2: return None

    filename = parameter_list[0]
    filecontent = ''.join(parameter_list[1:])

    return SleepAction(write_statusfile, filename, filecontent)

class SleepAction(SingleAction):
    pass